@@ -51,7 +51,10 @@ enum {
     DIAG_NV_MODE_PREF_HOME_ONLY       = 0x06,
     DIAG_NV_MODE_PREF_1X_ONLY         = 0x09,
     DIAG_NV_MODE_PREF_HDR_ONLY        = 0x0A,
-    DIAG_NV_MODE_PREF_1X_HDR_ONLY     = 0x0D,
+    DIAG_NV_MODE_PREF_GPRS_ONLY       = 0x0D,
+    DIAG_NV_MODE_PREF_UMTS_ONLY       = 0x0E,
+    DIAG_NV_MODE_PREF_GSM_UMTS_ONLY   = 0x11,
+    DIAG_NV_MODE_PREF_1X_HDR_ONLY     = 0x13,
     DIAG_NV_MODE_PREF_LTE_ONLY        = 0x1E,
     DIAG_NV_MODE_PREF_1X_HDR_LTE_ONLY = 0x24,
 };
